"""example:
python nilsson.4.py gd154p.in
where gd154p.in =  
0.0637  0.40  0.2  43.6994  1.0 # kappa,mu,eps2,lambda,Delta
154 6    # A,Nshells
0 1  # Nosc, z (proton=1, neutron=0
1 1  # etc
2 1
3 1
4 1
5 1

Output:  
basis ( nz,nr,m,sigma_z)
energies (index, Nshell, jz , E)
observables    
Npart   64.000
Nz,Nx,Ntot=  107.706  177.611  285.317 
Q_0, Np*<r^2> =   445.12   1577.65
"""

import os,sys
from utilities import *
from numpy import *
from numpy.linalg import eigh

def make_qnj(Nosc,jz,charge):
  qnjz = []
  for km in (jz-1,jz+1):
     m = km/2
     for nr in range(Nosc/2+1):
       for nz in range(Nosc+1):
         if ( m+nz + 2*nr) == Nosc :
           if jz > 2*m :
             qn = [nz,nr,m,1,charge]
           else:
             qn = [nz,nr,m,-1,charge]
           qnjz.append( qn)
  return qnjz
           
def levels(epsL,qnL):
   eps_dic = {}
   qn_all = []
   k= 0
   for i in range(len(epsL)) :
     qns = qnL[i]
     epss = epsL[i]
     for j in range(len(epss)):
       eps_dic[k] = epss[j]
       qn_all.append(qns[j])
       k += 1
   eps_sorted = []
   qn_sorted = []    
   for k in sorted(eps_dic, key=eps_dic.get):
      eps_sorted.append(eps_dic[k])
      qn_sorted.append(qn_all[k])
   print '\n index Nosc  Jz     E'
   for k in range(len(eps_sorted)):
     nz,nr,m,sigma_z,zch = qn_sorted[k]
     Nosc = nz+2*nr + m
     Jz = 2*m +sigma_z
     ss = '  %3i  %2i  %2i/2 %8.3f' % (k,Nosc,Jz,eps_sorted[k])
     print ss
      
def make_qnL(Nosc,charge):
  jzL = []
  NdimL = []
  qnL = []
  for jz in range(1,2*(Nosc+1)+1,2):
    print 'jz', jz
    jzL.append(jz)
    qnNjz  = make_qnj(Nosc,jz,charge)
    Ndim = len(qnNjz)
    NdimL.append(Ndim)
    qnL.append(qnNjz)
    for k in range(Ndim):
      print qnNjz[k]
    Njz = len(jzL)        
  return Njz,NdimL,qnL
  
def  make_orbs(A,Nosc,charge,params):
   Njz,NdimL,qnL  =  make_qnL(Nosc,charge)
   L2ave = Nosc*(Nosc+3)/2.
   kappa,mu,w0,wz,wxy = params
   L2mL =[]; LSmL = []; h0L = []; epsL= []; orbsL=[]
   for i in range(Njz) :
     qnJ= qnL[i]
     Ndim = NdimL[i]
     L2m = array([[0.0]*Ndim]*Ndim)
     LSm = L2m*1.0
     h0m = L2m*1.0
     hwm = L2m*1.0
     for k1 in range(Ndim) :
       hwm[k1,k1] = wz*(qnJ[k1][0]+0.5) + \
          wxy*(2*qnJ[k1][1]+qnJ[k1][2]+1.0)
       for k2 in range(k1+1) :
         LSm[k1,k2] = LS(qnJ[k1],qnJ[k2])
         LSm[k2,k1] = LSm[k1,k2]
         L2m[k1,k2] = L2(qnJ[k1],qnJ[k2])
         if k1 == k2 :
           L2m[k1,k2] += -L2ave            
         L2m[k2,k1] = L2m[k1,k2]
     L2mL.append(L2m)  
     LSmL.append(LSm)
     LSm = LSmL[i]
     L2m = L2mL[i]
     h0m = hwm -kappa*w0*(LSm-mu*L2m)
     evals,vecs = eigh(h0m)
     epsL.append(evals)
     orbsL.append(vecs)    
   return NdimL,qnL,Njz,epsL,orbsL  

def bcs(Delta,Lam,epsL):
  v2L=[]
  nn=0.0; s1= 0.0; s2 = 0.0
  for i in range(len(epsL)):
    eps= epsL[i]
    v2 = array([0.0]*len(eps))
    for k in range(len(eps)):
      Eqp = ((eps[k] -Lam)**2 + Delta**2)**0.5
      v2[k] = 0.5*(1-(eps[k] - Lam)/Eqp)
      nn += 2*v2[k]
      s1 += 1./Eqp
      s2 += 1.0-eps[k]/Eqp
    v2L.append(v2)
#  Lam_new =  (NN - s2)/s1
  return nn,v2L    #   ,Lam_new  


if __name__ == '__main__':
  inputf = sys.argv[1]
  print inputf
  lines = open(inputf).readlines()
  print lines[0]
  ss = lines[0].split();  
  kappa,mu,eps2,Lam0,Delta = map(float,ss[:5])
  print kappa,mu,eps2,Lam0,Delta
  ss = lines[1].split()
  A = float(ss[0]); Nshell = int(ss[1])
#  Delta = 1.
  ss = ' A=%3i Number of shells= %i \n' % (A,Nshell)
  print ss,
  w0 = 41.0/(A*1.0)**(1./3.)
  wz = w0*(1.0-2.*eps2/3.)
  wxy = w0*(1.0+eps2/3.)
  params = (kappa,mu,w0,wz,wxy)
  ss = ' eps2,w0,wz,wxy %4.2f %8.5f  %8.5f  %8.5f \n' %(eps2,w0,wz,wxy)
  print ss,  
  NdimL = []
  qnL = []
  epsL = []
  orbsL = []
  print Nshell
  for ish in range(Nshell) :
      ss = lines[ish + 2].split() 
      print ish,ss; Nosc= int(ss[0]); charge=int(ss[1])
      ss = ' Nosc=%2i  charge=%i \n' % (Nosc,charge)
      print ss,
      t1,t2,t3,t4,t5 = make_orbs(A,Nosc,charge,params)
      NdimL = NdimL + t1
#      print NdimL
      qnL = qnL + t2
#      print qnL
      epsL = epsL + t4
#      print epsL
      orbsL = orbsL + t5

  levels(epsL,qnL)
  Npart,v2L = bcs(Delta,Lam0,epsL)
  Nblock=len(orbsL)
  Nz = 0.0
  Nxy = 0.0
  rsq = 0.0
  q0 = 0.0
  for i in range(Nblock):
    orbs = orbsL[i]; qn= qnL[i]; v2= v2L[i]
    for korb in range(NdimL[i]):
      for kbasis in range(NdimL[i]):
        prob= orbs[kbasis,korb]**2*v2[korb]
        rsq += 2*prob*41.5*((qn[kbasis][0]+0.5)/wz +(
           2*qn[kbasis][1]+qn[kbasis][2]+1.0)/wxy)
        q0 += 2*prob*41.5*((qn[kbasis][0]+0.5)/wz -
         (2*qn[kbasis][1]+qn[kbasis][2]+1.0)/(2*wxy))       
        Nz += 2*prob*(qn[kbasis][0]+0.5)
        Nxy += 2*prob*(2*qn[kbasis][1]+qn[kbasis][2]+1.0)
  Nxyz = Nz+Nxy
  print '\nNp=  %8.3f' % Npart
  print 'Nz,Nx,Ntot= %8.3f %8.3f %8.3f ' % (Nz,Nxy,Nxyz)
  print 'q_0, Np*<r^2> = %8.2f  %8.2f' % (q0,rsq)
